require 'json'
require 'shellwords'

def readfile(file)
  f = File.new(file, "r")
  return f.read
end

def get_rails(command)
  `su - protonet -c "cd  /home/protonet/dashboard/current && bundle exec rails runner -e production '#{command}' 2>/dev/null"`
end

def protonet_config
  config = {}

  # the system hostname, statically set to protonet
  config['hostname'] = "protonet"

  # publish_to_web_name
  config['node_name'] = readfile("/etc/hostname").gsub(/\n*$/,'')

  config['publish_to_web'] = {}

  begin
    config['publish_to_web']['name'] = JSON.parse(get_rails("puts JSON.generate(SystemPreferences.where(var: \\\"publish_to_web_name\\\").as_json)"))[0]['value']
  rescue
    config['publish_to_web']['name'] = config['node_name']
  end

  begin
    config['publish_to_web']['public_host'] = JSON.parse(get_rails("puts JSON.generate(SystemPreferences.where(var: \\\"public_host\\\").as_json)"))[0]['value']
  rescue
    config['publish_to_web']['public_host'] = "#{config['publish_to_web']['name']}.protonet.info"
  end

  config['publish_to_web']['ssh_identity'] = {}
  config['publish_to_web']['ssh_identity']['public'] = readfile("/home/protonet/dashboard/shared/config/protonet.d/proxy_dsa.pub").gsub(/\n*$/,'')
  config['publish_to_web']['ssh_identity']['private'] = readfile("/home/protonet/dashboard/shared/config/protonet.d/proxy_dsa").gsub(/\n*$/,'')

  config['ssh'] = {}
  config['ssh']['host_keys'] = {}

  # ssh / host_keys / rsa / private
  config['ssh']['host_keys']['rsa'] = {}
  config['ssh']['host_keys']['rsa']['private'] = readfile("/etc/ssh/ssh_host_rsa_key").gsub(/\n*$/,'')

  # ssh / host_keys / rsa / public
  config['ssh']['host_keys']['rsa']['public'] = readfile("/etc/ssh/ssh_host_rsa_key.pub").gsub(/\n*$/,'')

  # ssh / host_keys / dsa / private
  config['ssh']['host_keys']['dsa'] = {}
  config['ssh']['host_keys']['dsa']['private'] = readfile("/etc/ssh/ssh_host_dsa_key").gsub(/\n*$/,'')

  # ssh / host_keys / dsa / public
  config['ssh']['host_keys']['dsa']['public'] = readfile("/etc/ssh/ssh_host_dsa_key.pub").gsub(/\n*$/,'')

  # ssh / host_keys / ecdsa / private
  config['ssh']['host_keys']['ecdsa'] = {}
  config['ssh']['host_keys']['ecdsa']['private'] = readfile("/etc/ssh/ssh_host_dsa_key").gsub(/\n*$/,'')

  # ssh / host_keys / ecdsa / public
  config['ssh']['host_keys']['ecdsa']['public'] = readfile("/etc/ssh/ssh_host_dsa_key.pub").gsub(/\n*$/,'')

  # wifi
  begin
    config['wifi'] = JSON.parse(get_rails("puts JSON.generate(SystemPreferences.where(var: \\\"wifi\\\").as_json)"))[0]['value']
  rescue Exception => e
    config['wifi'] = {
     "mode" => "wlan0",
     "channel" => 1,
     "wlan0" => {
       "name" => config['node_name'][0,32],
       "password" => "Changeme!123",
       "sharing" => true,
       "ip" => "10.42.0.1"
     },
     "wlan1" => {
       "name" => "#{config['node_name'][0,23]} (public)",
       "password" => "",
       "sharing" => true,
       "ip" => "10.43.0.1"
     }
   }
  end

  # license
  config['license'] = readfile("/home/protonet/.protonet-license").gsub(/\n*$/,'')

  # ssl / key
  config['ssl'] = {}
  config['ssl']['key'] = readfile("/home/protonet/dashboard/current/config/ssl_server_cert.key").gsub(/\n*$/,'')

  # ssl / cert
  config['ssl']['cert'] = readfile("/home/protonet/dashboard/current/config/ssl_server_cert.crt").gsub(/\n*$/,'')

  # passwords / root
  config['passwords'] = {}
  config['passwords']['root'] = `grep ^root < /etc/shadow | cut -f2 -d:`.chop

  # passwords / protonet
  config['passwords']['protonet'] = `grep ^protonet < /etc/shadow | cut -f2 -d:`.chop

  # variable to note if we skip the options for the current interface (i.e. do not
  # write them to the config

  skip_options = false

  File.open("/etc/network/interfaces", "r") do |ifaces|
    network_interfaces = {}
    current_iface = {}
    ifaces.readlines.each do |line|
      case line 
      when /^[:space:]*$/
        next
      when /^#/
        next
      when /^\s*iface\s+([^\s]+)\s+([^\s]+)\s+([^\s]+)/
        # TODO: currently we ignore he wlan as well (wlan interface is configured in services/wifi)
        if ["lo", "wlan0", "wlan1"].include?($1)
          skip_options = true
          next
        end
        # new interface, no skipping of options
        skip_options = false
        current_iface = {}
        network_interfaces[$1] = current_iface
        current_iface["family"] = $2
        current_iface["method"] = $3
        next
      when /^\s*auto/
        
      else

        if skip_options
          next
        end
        
        case line 
          # static fields
        when /^\s*address\s*([^\s]+)/
          current_iface["address"] = $1
        when /^\s*netmask\s*([^\s]+)/
          current_iface["netmask"] = $1
        when /^\s*gateway\s*([^\s]+)/
          current_iface["gateway"] = $1
        when /^\s*dns-nameservers\s*([^\s].*)$/
          current_iface["dns-nameservers"] = $1.split(" ")
        when /^\s*mtu\s*([^\s]+)/
          current_iface["mtu"] = $1
        when /^\s*bridge-ports\s*([^\s]+)/
          current_iface["bridge-ports"] = $1
        when /^\s*pre-up\s*([^\s]+)/
          current_iface["pre-up"] = $1
        when /^\s*(post-)?up\s*([^\s]+)/
          current_iface["post-up"] = $2
        when /^\s*(pre-)?down\s*([^\s]+)/
          current_iface["pre-down"] = $2
        when /^\s*post-down\s*([^\s]+)/
          
          # dhcp fields
        when /^\s*hostname\s*([^\s]+)/
          current_iface["hostname"] = $1
          
          # bridge fields
        when /^\s*bridge_ports\s*([^\s]+)/
          current_iface["bridge_ports"] = $1.split(" ")
        when /^\s*bridge_stp\s*([^\s]+)/
          current_iface["bridge_stp"] = $1
        when /^\s*bridge_fd\s*([^\s]+)/
          current_iface["bridge_fd"] = $1
        when /^\s*bridge_maxwait\s*([^\s]+)/
          current_iface["bridge_maxwait"] = $1
          
        when /^\s*allow-/
          next
        when /^\s*mapping/
          next
        when /^\s*source/
          next
        else 
          puts "unparsed #{line}"
        end
      end
    end
    config['network'] = {}
    config['network']['interfaces'] = network_interfaces
  end
  config
end